@@ -949,6 +949,7 @@ struct __CUDA_ALIGN__(4) __nv_fp6x4_e3m2 {
      * Constructor from \p double4 vector data type,
      * relies on \p __NV_SATFINITE behavior for out-of-range values.
      */
+    __NV_SILENCE_DEPRECATION_BEGIN
     explicit __CUDA_HOSTDEVICE_FP6__ __nv_fp6x4_e3m2(const double4 f) {
         const double2 flo = {f.x, f.y};
         const double2 fhi = {f.z, f.w};
@@ -958,6 +959,36 @@ struct __CUDA_ALIGN__(4) __nv_fp6x4_e3m2 {
             __nv_cvt_double2_to_fp6x2(fhi, __NV_E3M2, cudaRoundNearest);
         __x = __internal_pack_u16x2_to_u32(rlo, rhi);
     }
+    __NV_SILENCE_DEPRECATION_END
+    /**
+     * \ingroup CUDA_MATH_FP6_MISC
+     * Constructor from \p double4_16a vector data type,
+     * relies on \p __NV_SATFINITE behavior for out-of-range values.
+     */
+    explicit __CUDA_HOSTDEVICE_FP6__ __nv_fp6x4_e3m2(const double4_16a f) {
+        const double2 flo = {f.x, f.y};
+        const double2 fhi = {f.z, f.w};
+        const __nv_fp6x2_storage_t rlo =
+            __nv_cvt_double2_to_fp6x2(flo, __NV_E3M2, cudaRoundNearest);
+        const __nv_fp6x2_storage_t rhi =
+            __nv_cvt_double2_to_fp6x2(fhi, __NV_E3M2, cudaRoundNearest);
+        __x = __internal_pack_u16x2_to_u32(rlo, rhi);
+    }
+
+    /**
+     * \ingroup CUDA_MATH_FP6_MISC
+     * Constructor from \p double4_32a vector data type,
+     * relies on \p __NV_SATFINITE behavior for out-of-range values.
+     */
+    explicit __CUDA_HOSTDEVICE_FP6__ __nv_fp6x4_e3m2(const double4_32a f) {
+        const double2 flo = {f.x, f.y};
+        const double2 fhi = {f.z, f.w};
+        const __nv_fp6x2_storage_t rlo =
+            __nv_cvt_double2_to_fp6x2(flo, __NV_E3M2, cudaRoundNearest);
+        const __nv_fp6x2_storage_t rhi =
+            __nv_cvt_double2_to_fp6x2(fhi, __NV_E3M2, cudaRoundNearest);
+        __x = __internal_pack_u16x2_to_u32(rlo, rhi);
+    }
 
 #if !defined(__CUDA_NO_FP6_CONVERSION_OPERATORS__)
     /* Widening converts */
@@ -1500,6 +1531,7 @@ struct __CUDA_ALIGN__(4) __nv_fp6x4_e2m3 {
      * Constructor from \p double4 vector data type,
      * relies on \p __NV_SATFINITE behavior for out-of-range values.
      */
+    __NV_SILENCE_DEPRECATION_BEGIN
     explicit __CUDA_HOSTDEVICE_FP6__ __nv_fp6x4_e2m3(const double4 f) {
         const double2 flo = {f.x, f.y};
         const double2 fhi = {f.z, f.w};
@@ -1509,6 +1541,36 @@ struct __CUDA_ALIGN__(4) __nv_fp6x4_e2m3 {
             __nv_cvt_double2_to_fp6x2(fhi, __NV_E2M3, cudaRoundNearest);
         __x = __internal_pack_u16x2_to_u32(rlo, rhi);
     }
+    __NV_SILENCE_DEPRECATION_END
+    /**
+     * \ingroup CUDA_MATH_FP6_MISC
+     * Constructor from \p double4_16a vector data type,
+     * relies on \p __NV_SATFINITE behavior for out-of-range values.
+     */
+    explicit __CUDA_HOSTDEVICE_FP6__ __nv_fp6x4_e2m3(const double4_16a f) {
+        const double2 flo = {f.x, f.y};
+        const double2 fhi = {f.z, f.w};
+        const __nv_fp6x2_storage_t rlo =
+            __nv_cvt_double2_to_fp6x2(flo, __NV_E2M3, cudaRoundNearest);
+        const __nv_fp6x2_storage_t rhi =
+            __nv_cvt_double2_to_fp6x2(fhi, __NV_E2M3, cudaRoundNearest);
+        __x = __internal_pack_u16x2_to_u32(rlo, rhi);
+    }
+
+    /**
+     * \ingroup CUDA_MATH_FP6_MISC
+     * Constructor from \p double4_32a vector data type,
+     * relies on \p __NV_SATFINITE behavior for out-of-range values.
+     */
+    explicit __CUDA_HOSTDEVICE_FP6__ __nv_fp6x4_e2m3(const double4_32a f) {
+        const double2 flo = {f.x, f.y};
+        const double2 fhi = {f.z, f.w};
+        const __nv_fp6x2_storage_t rlo =
+            __nv_cvt_double2_to_fp6x2(flo, __NV_E2M3, cudaRoundNearest);
+        const __nv_fp6x2_storage_t rhi =
+            __nv_cvt_double2_to_fp6x2(fhi, __NV_E2M3, cudaRoundNearest);
+        __x = __internal_pack_u16x2_to_u32(rlo, rhi);
+    }
 
 #if !defined(__CUDA_NO_FP6_CONVERSION_OPERATORS__)
     /* Widening converts */
