@@ -111,7 +111,18 @@ extern "C"
 	CUFILE_OP(CUFILEOP_BASE_ERR + 36, CU_FILE_GPU_MEMORY_PINNING_FAILED, failed to allocate pinned GPU Memory) \
 	CUFILE_OP(CUFILEOP_BASE_ERR + 37, CU_FILE_BATCH_FULL, queue full for batch operation) \
 	CUFILE_OP(CUFILEOP_BASE_ERR + 38, CU_FILE_ASYNC_NOT_SUPPORTED, cuFile stream operation not supported) \
-	CUFILE_OP(CUFILEOP_BASE_ERR + 39, CU_FILE_IO_MAX_ERROR, GPUDirect Storage Max Error)
+	CUFILE_OP(CUFILEOP_BASE_ERR + 39, CU_FILE_INTERNAL_BATCH_SETUP_ERROR, batch setup internal error - retry later) \
+	CUFILE_OP(CUFILEOP_BASE_ERR + 40, CU_FILE_INTERNAL_BATCH_SUBMIT_ERROR, batch submit internal error - retry later) \
+	CUFILE_OP(CUFILEOP_BASE_ERR + 41, CU_FILE_INTERNAL_BATCH_GETSTATUS_ERROR, batch get status internal error - retry later) \
+	CUFILE_OP(CUFILEOP_BASE_ERR + 42, CU_FILE_INTERNAL_BATCH_CANCEL_ERROR, batch cancel internal error - retry later) \
+	CUFILE_OP(CUFILEOP_BASE_ERR + 43, CU_FILE_NOMEM_ERROR, cufile no memory error - retry later) \
+	CUFILE_OP(CUFILEOP_BASE_ERR + 44, CU_FILE_IO_ERROR, cufile io error) \
+	CUFILE_OP(CUFILEOP_BASE_ERR + 45, CU_FILE_INTERNAL_BUF_REGISTER_ERROR, cufile buf registration error) \
+	CUFILE_OP(CUFILEOP_BASE_ERR + 46, CU_FILE_HASH_OPR_ERROR, cufile hash operation error) \
+	CUFILE_OP(CUFILEOP_BASE_ERR + 47, CU_FILE_INVALID_CONTEXT_ERROR, cufile invalid context error) \
+	CUFILE_OP(CUFILEOP_BASE_ERR + 48, CU_FILE_NVFS_INTERNAL_DRIVER_ERROR, nvfs internal driver error) \
+	CUFILE_OP(CUFILEOP_BASE_ERR + 49, CU_FILE_BATCH_NOCOMPAT_ERROR, compat mode off error) \
+	CUFILE_OP(CUFILEOP_BASE_ERR + 50, CU_FILE_IO_MAX_ERROR, GPUDirect Storage Max Error)
 
 
 /**
@@ -318,7 +329,7 @@ typedef void* CUfileHandle_t;
  * <b>Description</b>
  * cuFileHandleRegister registers the open file descriptor for use with cuFile IO operations.
  *
- * This API will ensure that the file’s descriptor is checked for GPUDirect Storage support and returns a valid file handle on CU_FILE_SUCCESS.
+ * This API will ensure that the file's descriptor is checked for GPUDirect Storage support and returns a valid file handle on CU_FILE_SUCCESS.
  *
  * @note the file needs to be opened in O_DIRECT mode to support GPUDirect Storage.
  *
@@ -766,6 +777,11 @@ typedef enum CUFileStringConfigParameter_t {
 	CUFILE_PARAM_LOG_DIR,
 } CUFileStringConfigParameter_t;
 
+typedef enum CUFileArrayConfigParameter_t {
+	CUFILE_PARAM_POSIX_POOL_SLAB_SIZE_KB,
+	CUFILE_PARAM_POSIX_POOL_SLAB_COUNT,
+} CUFileArrayConfigParameter_t;
+
 
 // GET Parameter API
 /**
@@ -786,6 +802,17 @@ CUfileError_t cuFileGetParameterSizeT(CUFileSizeTConfigParameter_t param, size_t
 CUfileError_t cuFileGetParameterBool(CUFileBoolConfigParameter_t param, bool *value);	
 CUfileError_t cuFileGetParameterString(CUFileStringConfigParameter_t param, char *desc_str, int len);	
 
+/**
+ * @brief Get both the minimum and maximum settable values for a given size_t parameter in a single call
+ *
+ * @param param CUfile SizeT configuration parameter
+ * @param min_value Pointer to store the minimum value
+ * @param max_value Pointer to store the maximum value
+ * @return CUfileError_t Returns CU_FILE_SUCCESS on success
+ *                       Returns CU_FILE_INVALID_VALUE if min_value or max_value is NULL
+ */
+CUfileError_t cuFileGetParameterMinMaxValue(CUFileSizeTConfigParameter_t param, size_t *min_value, size_t *max_value);
+
 // SET Parameter APIs
 /**
  *@brief
@@ -806,6 +833,276 @@ CUfileError_t cuFileSetParameterSizeT(CUFileSizeTConfigParameter_t param, size_t
 CUfileError_t cuFileSetParameterBool(CUFileBoolConfigParameter_t param, bool value);
 CUfileError_t cuFileSetParameterString(CUFileStringConfigParameter_t param, const char* desc_str);
 
+/**
+ * @brief Set the level of statistics collection for cuFile operations. This will override 
+ * the cufile.json settings for stats.
+ *
+ * @param level Statistics level (0 = disabled, 1 = basic, 2 = detailed, 3 = verbose)
+ *
+ * @return CU_FILE_SUCCESS on success
+ * @return CU_FILE_INVALID_VALUE if level is invalid
+ *
+ * @note Higher stats levels may impact performance. Level 0 disables statistics.
+ * @note Changes to stats level take effect for future operations.
+ *
+ * @see cuFileGetStatsLevel
+ */
+CUfileError_t cuFileSetStatsLevel(int level);
+
+/**
+ * @brief Get the current level of statistics collection for cuFile operations
+ *
+ * @param level Pointer to store the current statistics level
+ *
+ * @return CU_FILE_SUCCESS on success
+ * @return CU_FILE_INVALID_VALUE if level is NULL
+ *
+ * @see cuFileSetStatsLevel
+ */
+CUfileError_t cuFileGetStatsLevel(int *level);
+
+#define CUFILE_GPU_UUID_LEN 16
+/**
+ * @brief Counter structure for tracking operation successes and failures
+ */
+typedef struct CUfileOpCounter {
+    uint64_t ok;   // Number of successful operations
+    uint64_t err;  // Number of failed operations
+} CUfileOpCounter_t;
+
+/**
+ * @brief Level 1 Statistics: Basic IO and operation counters
+ */
+typedef struct CUfileStatsLevel1 {
+    // Operation counters
+    CUfileOpCounter_t read_ops;              // Read operations
+    CUfileOpCounter_t write_ops;             // Write operations
+    CUfileOpCounter_t hdl_register_ops;      // Handle register operations
+    CUfileOpCounter_t hdl_deregister_ops;    // Handle deregister operations
+    CUfileOpCounter_t buf_register_ops;      // Buffer register operations
+    CUfileOpCounter_t buf_deregister_ops;    // Buffer deregister operations
+    
+    // Basic IO statistics
+    uint64_t read_bytes;               // Total bytes read
+    uint64_t write_bytes;              // Total bytes written
+    uint64_t read_bw_bytes_per_sec;            // Read bandwidth (bytes/sec)
+    uint64_t write_bw_bytes_per_sec;           // Write bandwidth (bytes/sec)
+    uint64_t read_lat_avg_us;          // Average read latency (microseconds)
+    uint64_t write_lat_avg_us;         // Average write latency (microseconds)
+    
+    // Operations per second
+    uint64_t read_ops_per_sec;         // Read operations per second
+    uint64_t write_ops_per_sec;        // Write operations per second
+    
+    // Latency sums
+    uint64_t read_lat_sum_us;          // Sum of read latencies
+    uint64_t write_lat_sum_us;         // Sum of write latencies
+    
+    // Batch operations counters
+    CUfileOpCounter_t batch_submit_ops;      // Batch submit operations
+    CUfileOpCounter_t batch_complete_ops;    // Batch complete operations
+    CUfileOpCounter_t batch_setup_ops;       // Batch setup operations
+    CUfileOpCounter_t batch_cancel_ops;      // Batch cancel operations
+    CUfileOpCounter_t batch_destroy_ops;     // Batch destroy operations
+    
+    // Batch queue counters
+    CUfileOpCounter_t batch_enqueued_ops;    // Batch enqueue operations
+    CUfileOpCounter_t batch_posix_enqueued_ops;  // POSIX batch enqueue operations
+    CUfileOpCounter_t batch_processed_ops;    // Batch process operations
+    CUfileOpCounter_t batch_posix_processed_ops;  // POSIX batch process operations
+    
+    // Batch submission type counters
+    CUfileOpCounter_t batch_nvfs_submit_ops;   // NVFS batch submit operations
+    CUfileOpCounter_t batch_p2p_submit_ops;    // P2P batch submit operations
+    CUfileOpCounter_t batch_aio_submit_ops;    // AIO batch submit operations
+    CUfileOpCounter_t batch_iouring_submit_ops; // IO_URING batch submit operations
+    CUfileOpCounter_t batch_mixed_io_submit_ops; // Mixed IO batch submit operations
+    CUfileOpCounter_t batch_total_submit_ops;   // Total batch submit operations
+    
+    // Batch operations statistics
+    uint64_t batch_read_bytes;         // Total batch read bytes
+    uint64_t batch_write_bytes;        // Total batch write bytes
+    uint64_t batch_read_bw_bytes;      // Batch read bandwidth
+    uint64_t batch_write_bw_bytes;     // Batch write bandwidth
+    uint64_t batch_submit_lat_avg_us;  // Avg batch submit latency
+    uint64_t batch_completion_lat_avg_us; // Avg batch completion latency
+    uint64_t batch_submit_ops_per_sec;  // Batch submit operations per second
+    uint64_t batch_complete_ops_per_sec; // Batch complete operations per second
+    uint64_t batch_submit_lat_sum_us;    // Sum of batch submit latencies
+    uint64_t batch_completion_lat_sum_us; // Sum of batch completion latencies
+    uint64_t last_batch_read_bytes;      // Last batch read bytes
+    uint64_t last_batch_write_bytes;     // Last batch write bytes
+} CUfileStatsLevel1_t;
+
+/**
+ * @brief Level 2 Statistics: Includes Level 1 plus size histograms and detailed metrics
+ */
+typedef struct CUfileStatsLevel2 {
+    // Basic statistics (Level 1)
+    CUfileStatsLevel1_t basic;
+    
+    // IO size histograms
+    uint64_t read_size_kb_hist[32];    // Histogram of read sizes
+    uint64_t write_size_kb_hist[32];   // Histogram of write sizes
+} CUfileStatsLevel2_t;
+
+/**
+ * @brief Per-GPU statistics structure used in Level 3
+ */
+typedef struct CUfilePerGpuStats {
+    char uuid[CUFILE_GPU_UUID_LEN];       // GPU UUID
+    
+    // Read operations
+    uint64_t read_bytes;           // Total bytes read
+    uint64_t read_bw_bytes_per_sec;// Read bandwidth in bytes per second
+    uint64_t read_utilization;     // Read utilization percentage
+    uint64_t read_duration_us;     // Read operation duration
+    uint64_t n_total_reads;        // Total number of reads
+    uint64_t n_p2p_reads;          // Number of PCIe P2PDMA reads
+    uint64_t n_nvfs_reads;         // Number of nvidia-fs reads
+    uint64_t n_posix_reads;        // Number of POSIX reads
+    uint64_t n_unaligned_reads;    // Number of unaligned reads
+    uint64_t n_dr_reads;           // Number of reads using dynamic routing
+    uint64_t n_sparse_regions;     // Number of sparse regions
+    uint64_t n_inline_regions;     // Number of inline regions
+    uint64_t n_reads_err;          // Number of read errors
+    
+    // Write operations
+    uint64_t writes_bytes;         // Total bytes written
+    uint64_t write_bw_bytes_per_sec;// Write bandwidth in bytes per secind
+    uint64_t write_utilization;    // Write utilization percentage  
+    uint64_t write_duration_us;    // Write operation duration
+    uint64_t n_total_writes;       // Total number of writes
+    uint64_t n_p2p_writes;      // Number of PCIe P2PDMA writes
+    uint64_t n_nvfs_writes;        // Number of nvidia-fs writes
+    uint64_t n_posix_writes;       // Number of POSIX writes
+    uint64_t n_unaligned_writes;   // Number of unaligned writes
+    uint64_t n_dr_writes;          // Number of writes using dynamic routing
+    uint64_t n_writes_err;         // Number of write errors
+    
+    // Buffer registration statistics
+    uint64_t n_mmap;               // Number of buffer registrations
+    uint64_t n_mmap_ok;            // Successful registrations
+    uint64_t n_mmap_err;           // Failed registrations
+    uint64_t n_mmap_free;          // Number of buffer deregistrations
+    uint64_t reg_bytes;            // Total bytes registered
+} CUfilePerGpuStats_t;
+
+/**
+ * @brief Level 3 Statistics: Includes Level 2 plus per-GPU and subsystem statistics
+ */
+typedef struct CUfileStatsLevel3 {
+    // Detailed statistics (Level 2)
+    CUfileStatsLevel2_t detailed;
+    
+    // Number of GPUs detected
+    uint32_t num_gpus;
+    
+    // Per-GPU statistics (array for each GPU)
+    CUfilePerGpuStats_t per_gpu_stats[16]; // Using the maxGpus constant value from cufile_stats.h
+} CUfileStatsLevel3_t;
+
+/**
+ * @brief Start collecting cuFile statistics
+ *
+ * @return CU_FILE_SUCCESS on success
+ * @return CU_FILE_DRIVER_NOT_INITIALIZED if the driver is not initialized
+ * 
+ * @note Statistics level must be set using cuFileSetStatsLevel before calling this function
+ */
+CUfileError_t cuFileStatsStart(void);
+
+/**
+ * @brief Stop collecting cuFile statistics
+ *
+ * @return CU_FILE_SUCCESS on success
+ * @return CU_FILE_DRIVER_NOT_INITIALIZED if the driver is not initialized
+ */
+CUfileError_t cuFileStatsStop(void);
+
+/**
+ * @brief Reset all cuFile statistics counters
+ *
+ * @return CU_FILE_SUCCESS on success
+ * @return CU_FILE_DRIVER_NOT_INITIALIZED if the driver is not initialized
+ */
+CUfileError_t cuFileStatsReset(void);
+
+/**
+ * @brief Get Level 1 cuFile statistics
+ *
+ * @param stats Pointer to CUfileStatsLevel1_t structure to be filled
+ *
+ * @return CU_FILE_SUCCESS on success
+ * @return CU_FILE_INVALID_VALUE if stats is NULL or level 1 stats not enabled
+ * @return CU_FILE_DRIVER_NOT_INITIALIZED if the driver is not initialized
+ */
+CUfileError_t cuFileGetStatsL1(CUfileStatsLevel1_t *stats);
+
+/**
+ * @brief Get Level 2 cuFile statistics
+ *
+ * @param stats Pointer to CUfileStatsLevel2_t structure to be filled
+ *
+ * @return CU_FILE_SUCCESS on success
+ * @return CU_FILE_INVALID_VALUE if stats is NULL or level 2 stats not enabled
+ * @return CU_FILE_DRIVER_NOT_INITIALIZED if the driver is not initialized
+ */
+CUfileError_t cuFileGetStatsL2(CUfileStatsLevel2_t *stats);
+
+/**
+ * @brief Get Level 3 cuFile statistics
+ *
+ * @param stats Pointer to CUfileStatsLevel3_t structure to be filled
+ *
+ * @return CU_FILE_SUCCESS on success
+ * @return CU_FILE_INVALID_VALUE if stats is NULL or level 3 stats not enabled
+ * @return CU_FILE_DRIVER_NOT_INITIALIZED if the driver is not initialized
+ */
+CUfileError_t cuFileGetStatsL3(CUfileStatsLevel3_t *stats);
+
+/*
+ * Get the BAR size for a specific GPU
+ *
+ * @param gpuIndex : GPU index to query
+ * @param barSize  : Pointer to store the BAR size in MiB
+ *
+ * @returns: CU_FILE_SUCCESS on success or 
+ *           CU_FILE_DRIVER_NOT_INITIALIZED, CU_FILE_INVALID_VALUE on error
+ */
+CUfileError_t cuFileGetBARSizeInKB(int gpuIndex, size_t *barSize);
+
+/**
+ * @brief Set both POSIX pool slab size and count parameters as a pair
+ *
+ * @param size_values Array of slab sizes in KB
+ * @param count_values Array of slab counts
+ * @param len Length of both arrays (must be the same)
+ *
+ * @return CU_FILE_SUCCESS on success
+ * @return CU_FILE_INVALID_VALUE if inputs are invalid
+ * @return CU_FILE_DRIVER_ALREADY_OPEN if driver is already open
+ */
+CUfileError_t cuFileSetParameterPosixPoolSlabArray(
+                                     const size_t *size_values,
+                                     const size_t *count_values,
+                                     int len);
+
+/**
+ * @brief Get both POSIX pool slab size and count parameters as a pair
+ *
+ * @param size_values Buffer to receive slab sizes in KB
+ * @param count_values Buffer to receive slab counts
+ * @param len Buffer size (must match the actual parameter length)
+ *
+ * @return CU_FILE_SUCCESS on success
+ * @return CU_FILE_INVALID_VALUE if inputs are invalid or buffer size doesn't match
+ */
+CUfileError_t cuFileGetParameterPosixPoolSlabArray(
+                                     size_t *size_values,
+                                     size_t *count_values,
+                                     int len);
+
 #pragma GCC visibility pop
 
 /// @cond DOXYGEN_SKIP_MACRO
